@@ -21,15 +21,14 @@
 #include "config.h"
 #endif
 
-#include "libbluray/bluray.h"
-#include "util/macro.h"
-#include "util/strutl.h"
-
+#include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <libgen.h>
 #include <inttypes.h>
 
+#include "libbluray/bluray.h"
+
 #define PKT_SIZE 192
 #define BUF_SIZE (PKT_SIZE * 1024)
 #define MIN(a,b) (((a) < (b)) ? a : b)
