@@ -250,10 +250,11 @@ public class HScene extends Container implements HComponentOrdering {
             removeAll();
 
             Graphics g = GUIManager.getInstance().getGraphics();
-            Rectangle r = getBounds();
-            g.clearRect(r.x, r.y, r.width, r.height);
-            g.dispose();
-
+            if (g != null) {
+                Rectangle r = getBounds();
+                g.clearRect(r.x, r.y, r.width, r.height);
+                g.dispose();
+            }
             if (image != null) {
                 image.flush();
             }
@@ -318,11 +319,11 @@ public class HScene extends Container implements HComponentOrdering {
     }
 
     public int[] getAllShortcutKeycodes() {
-        Integer[] src = (Integer[]) shortcuts.keySet().toArray();
+        Object[] src = shortcuts.keySet().toArray();
         int[] dest = new int[src.length];
 
         for (int i = 0; i < src.length; i++)
-            dest[i] = src[i].intValue();
+            dest[i] = ((Integer)src[i]).intValue();
 
         return dest;
     }
